package module_standard.wSerial;

import persistance.wGlobalParam.Gp;
import a_doc.doc.Licence;
/**
<br>{@link A_doc}
 <br>{@link Licence}.
 */
public abstract class AbstractMS1204  extends Thread {
	private static final long serialVersionUID = 1L;
	Serial serial=null;
	private boolean waitCodeCb=false;
	int envParam=0xC6;
	int cbOpen=0;
	public String codeCb;
	public AbstractMS1204(Serial serial){
		this.serial=serial;
	}
	public void run(){
		if (!serial.isOpenTimeOut()){
			cbOpen=9;
		} else {
			if (serial.isOpen()){
				cbOpen=2;
				initRxTx();
				initCB();
			} else {
				cbOpen=9;
			}
		}
	}
	public boolean isCbInitInit(){
		boolean ret=true;
		if (cbOpen!=0){
			ret=false;
		}
		return ret;
	}
	public boolean isCbOpen(){
		boolean ret=false;
		if (cbOpen==2){
			ret=true;
		}
		return ret;
	}
	protected abstract boolean initCB();
	private void initRxTx(){
		serial.setRTS(false);
		serial.setDTR(true);
		try {  // Laisse du temps pour que la trame arrive
			Thread.sleep(20);
		} catch (InterruptedException arg0) { arg0.printStackTrace(); } 
	}
	public boolean sendTx(int opCode,int[] mess){
		boolean ret=false;
		if (isWaitCodeCb()==true){ //Libre l'attente de lecture de code pour passer des paramtres au CB
			setWaitCodeCb(false);
		}
		serial.setAckControl(Serial.ACKCONTROL_BYTE);
		serial.setRTS(true);
		while (true){
			try {  // Laisse du temps pour la rponse au rts
				Thread.sleep(20);
			} catch (InterruptedException arg0) { arg0.printStackTrace(); } 
			if (serial.isCTS()){
				serial.setRTS(false);
				break;
			}
		}
		int source=0x04;
		int status=0x00;
		int cs=0;
		cs=outLENGTH(mess,4,cs);
		cs=outOPTCODE(opCode,cs);
		cs=outSOURCE(source,cs);
		cs=outSTATUS(status,cs);
		cs=outMESS(mess,cs);
		outCHECKSUM(cs);
		ret=checkAckProtocol();
		return ret;
	}
	private boolean checkAckProtocol(){
		boolean ret=false;
		serial.setAckPF(false);
		while (serial.isAckPF()==false){
			try {  
				Thread.sleep(getSleepSerial());
			} catch (InterruptedException arg0) { arg0.printStackTrace(); } 
		}
		int tmp=serial.getBTrame()[1] & 0xff;
		if (tmp==208){
			ret=true;
		} else if (tmp==209){
			Gp.MB("Trame non reconnu, Lecteur Code Barre sur : "+serial.portCom);
			ret=false;
		} else {
			Gp.MB("Erreur communication Lecteur Code Barre sur : "+serial.portCom);
			ret=false;
		}
		return ret;
	}
	public void setDebug(boolean enable){
		serial.setDebug(enable);
	}
	public void close(){
		serial.close();
		serial=null;
	}
	private void outCHECKSUM(int cs) {
		cs=(256*256)-cs;
		serial.write(cs/256);
		serial.write(cs%256);
	}
	private int outMESS(int[] mess, int cs) {
		for(int i=0;i<mess.length;i++){
			serial.write(mess[i]);
			cs=cs+mess[i];
		}
		return cs;
	}
	private int outSTATUS(int status, int cs) {
		serial.write(status);
		cs=cs+status;
		return cs;
	}
	private int outSOURCE(int source, int cs) {
		serial.write(source);
		cs=cs+source;
		return cs;
	}
	private int outOPTCODE(int opCode, int cs) {
		serial.write(opCode);
		cs=cs+opCode;
		return cs;
	}
	private int outLENGTH(int[] mess, int lenfixe, int cs) {
		serial.write(mess.length+lenfixe);
		cs=cs+mess.length+lenfixe;
		return cs;
	}
	protected boolean enable2TO5(boolean beep){
		boolean ret=true;
		if (ret){
			ret=sendTx(envParam,new int[]{getBeep(false),0x06,0x01});//enable interleave 2 of 5 
		}
		if (ret){
			ret=sendTx(envParam,new int[]{getBeep(false),0xEB,0x05}); // Scan data Transmission 
		}
		if (ret){
			ret=sendTx(envParam,new int[]{getBeep(false),0xEB,0x07}); // Scan data Transmission 2 
		}
		return ret;
	}
	public void setParamTrame(int prefix,int length,int suffix1,int suffix2) {
		boolean ret=true;
		if (ret){
			sendTx(envParam,new int[]{getBeep(false),0x69,prefix});//
		}
		if (ret){
			sendTx(envParam,new int[]{getBeep(false),0x68,suffix1});//
		}
		if (ret){
			sendTx(envParam,new int[]{getBeep(false),0x6A,suffix2});//
		}
		if (ret){
			sendTx(envParam,new int[]{getBeep(false),0x16,length});//
		}
		if (ret){
			serial.setTrameParams(prefix,length,suffix1,suffix2);
		}
	}

	public boolean waitCodeCb(){
		boolean ret=false;
		if (isWaitCodeCb()==false){
			setWaitCodeCb(true);
			serial.setAckControl(Serial.ACKCONTROL_STX_BYTES_ETX1_ETX2);{
				serial.setAckPF(false);
				while (serial.isAckPF()==false && isWaitCodeCb()==true ){
					try {  
						Thread.sleep(getSleepSerial());
					} catch (InterruptedException arg0) { arg0.printStackTrace(); } 
				}
				if (isWaitCodeCb()==true){
					codeCb=serial.getSTrame();
				} else {
					codeCb=null;
				}
			}
			//System.out.println("serial :"+codeCb);
			stopWaitCodeCb();
			ret=true;
		}
		return ret;
	}
	public void stopWaitCodeCb() {
		setWaitCodeCb(false);
	}
	public String readCodeCb() {
		String ret=codeCb;
		codeCb=null;
		return ret;
	}

	public boolean setCbOn(boolean beep){
		boolean ret=sendTx(envParam,new int[]{getBeep(beep),0x8A,0x04});// Continious
		return ret;
	}
	public boolean setCbOff(boolean beep){
		boolean ret=sendTx(envParam,new int[]{getBeep(beep),0x8A,0x00});// host
		return ret;
	}
	protected boolean cbBeep(){
		boolean ret=sendTx(0xE6,new int[]{0x14});// Beep
		return ret;
	}
	private int getBeep(boolean beep){
		int ret;
		if (beep){
			ret=0x00; // beep
		} else {
			ret=0xFF; // no beep
		}
		return ret;
	}
	public long getSleepSerial(){
		return serial.getSleep()/2;
	}
	public void setWaitCodeCb(boolean waitCodeCb) {
		this.waitCodeCb = waitCodeCb;
	}
	public boolean isWaitCodeCb() {
		return waitCodeCb;
	}
}


